/***************************************************************************
                         eval.cpp  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file eval.cpp
  \brief Implementation of the class definition of 'eval'

   Contains the header of the class 'eval', which takes care of OT style evaluation
   of candidate sets on the basis of constraint rankings and constraint violation patterns.
*/

#include "eval.h"

int Eval::BestOutput(int input, 
		     std::vector<int> &Ranking, 
		     std::vector<int> &ViolationPattern, 
		     std::vector<int> &GEN)
{
  int o, output=0;
  for (o=0; o<NOutputs;o++) {
    if (IsBetterOutput(input, o, output, Ranking, ViolationPattern, GEN) >0) {
      output = o;
    }
  }
  return output;
}


int Eval::IsBetterOutput(int i, 
			 int o1, 
			 int o2, 
			 std::vector<int> &Ranking, 
			 std::vector<int> &ViolationPattern, 
			 std::vector<int> &GEN)
{
  int r, c, diff, v=0;
  
  
  if (bi_eval) {
    r = (IsOptimalInput(i, o1, Ranking, ViolationPattern, GEN) -
	 IsOptimalInput(i, o2, Ranking, ViolationPattern, GEN));
    if (r != 0) return r;
  }
  
  
  if (GEN[i*NOutputs+o1] != -1 && GEN[i*NOutputs+o2] == -1) return 1;
  if (GEN[i*NOutputs+o1] == -1 && GEN[i*NOutputs+o2] != -1) return -1;
  
  
  for (c = NConstraints-1; c >= 0 && v==0; c--) {
    diff = ViolationPattern[(i*NOutputs+o2)*NConstraints+Ranking[c]] -
      ViolationPattern[(i*NOutputs+o1)*NConstraints+Ranking[c]];
    if (diff > 0) return 1;
    if (diff < 0) return -1;
  }
  return 0;
}

int Eval::IsOptimalInput(int i, 
			 int o, 
			 std::vector<int> &Ranking, 
			 std::vector<int> &violations, 
			 std::vector<int> &GEN)
{
  int bi;
  for (bi = (i % NEquivalenceClasses); bi < NInputs; bi += NEquivalenceClasses) {
    if (bi != i && IsBetterInput(bi, i, o, Ranking, violations, GEN) > 0) return 0;
  }
  return 1;
}

int Eval::BestInput(int output, 
		    std::vector<int> &Ranking, 
		    std::vector<int> &ViolationPattern, 
		    std::vector<int> &GEN)
{
  int i, input=0;
  for (i=0; i<NInputs;i++) {
    if (IsBetterInput(i, input, output, Ranking, ViolationPattern, GEN) >=0) input = i;
  }
  return input;
}

int Eval::IsBetterInput(int i1, 
			int i2, 
			int o, 
			std::vector<int> &Ranking, 
			std::vector<int> &ViolationPattern, 
			std::vector<int> &GEN)
{
  int diff,c,v=0;
  
  if (GEN[i1*NOutputs+o] != -1 && GEN[i2*NOutputs+o] == -1) return 1;
  if (GEN[i1*NOutputs+o] == -1 && GEN[i2*NOutputs+o] != -1) return -1;
  for (c=NConstraints-1;c>=0&&v==0; c--) {
    diff = ViolationPattern[(i2*NOutputs+o)*NConstraints+Ranking[c]] -
      ViolationPattern[(i1*NOutputs+o)*NConstraints+Ranking[c]];
    if (diff > 0) v = 1;
    if (diff < 0) v = -1;
  }
  return v;
}
