/***************************************************************************
                         eval.h  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file eval.h
  \brief Header of the class definition of 'eval'

   Contains the implementation of the class 'eval', which takes care of OT style evaluation
   of candidate sets on the basis of constraint rankings and constraint violation patterns.
*/
#ifndef EVAL_H_
#define EVAL_H_

#include <vector>
//using namespace std;


class Eval 
{
 public:
  int NInputs;
  int NOutputs;
  int NConstraints;
  int NEquivalenceClasses;
  
  //! if 'true', optimality is bidirectional, else unidirectional
  bool bi_eval;
  
 public:
  //! returns the best output for 'input'
  /*! on the basis of 'Ranking', 'ViolationPattern', and 'GEN'
   */
  int BestOutput(int input, 
	  std::vector<int> &Ranking, 
	  std::vector<int> &ViolationPattern,
	  std::vector<int> &GEN);

  //! returns 1 if (i,o1) is better than (i,o2)
  /*! on the basis of 'Ranking', 'ViolationPattern', an 'GEN'. If (i,o2) is better
      than (i,o1), it returns -1; if both have the same violation profile, 0 is returned.
  */
  int IsBetterOutput(int i, 
		     int o1, 
		     int o2, 
		     std::vector<int> &Ranking, 
		     std::vector<int> &ViolationPattern, 
		     std::vector<int> &GEN);

  //! returns the optimal input for output 'output'
  int BestInput(int output, 
		std::vector<int> &Ranking, 
		std::vector<int> &ViolationPattern, 
		std::vector<int> &GEN);

  //!returns 1 if (i1,o) is better than (i2,o)
  /*! on the basis of 'Ranking', 'ViolationPattern', an 'GEN'. If (i2,o) is better 
      than (i1,o), it returns -1; if both have the same violation profile, 0 is returned.
  */
  int IsBetterInput(int i1, 
		    int i2, 
		    int o, 
		    std::vector<int> &Ranking, 
		    std::vector<int> &ViolationPattern, 
		    std::vector<int> &GEN);

  //! returns 1 if 'i' is the optimal input for the output 'o', 0 else
  int IsOptimalInput(int i, 
		     int o, 
		     std::vector<int> &Ranking, 
		     std::vector<int> &violations, 
		     std::vector<int> &GEN);
}
;

#endif

