/***************************************************************************
                      evolot_run.cpp  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file evolot_run.cpp
   \brief Implementation of the 'Evolotrun' class

   Contains the implementation of the 'Evolotrun::run()' procedure
   alongside with some auxiliary c-style function definitions.
*/

#include "evolot_run.h"

void Evolotrun::run()
{
	bool IsWindows=false;
  QApplication::setOverrideCursor (Qt::waitCursor);
  
  int i, i1, j, o1, c, c1, n, g;
  vector<double> value(NConstraints);
  int observation[2];
  vector<int> Ranking(NConstraints), TestOutput(NInputs*NOutputs), 
    violates(NInputs*NOutputs*NConstraints);
  vector<int> GEN(NInputs*NOutputs);

  Eval eval;
  eval.NInputs = NInputs;
  eval.NOutputs = NOutputs;
  eval.NConstraints = NConstraints;
  eval.NEquivalenceClasses = NEquivalenceClasses;
  eval.bi_eval = bi_eval;
  ActualValues.resize(NConstraints);
  
  srand((unsigned int)time((time_t *)NULL)); // Initialization of random number generator  

  ofstream con_output(con_outputfile, ios::out);

  GetGenerator(genfile, GEN);

  GetViolationPattern(starfile, violates);
  
  ofstream genout(gen_outputfile);
  
  lcd->show();
  
  table->setSelectionMode(table->NoSelection);
  table->setNumRows(NInputs);
  table->setNumCols(NOutputs);
  int height=0, width = 0, ti=0, to=0;
  for (ti=0;ti<NInputs;ti++) {
    width = 0;
    for (to=0;to<NOutputs;to++) {
      //QString s;
      table->setText(ti,to, "          ");
      table->adjustColumn(to);
      table->resize(100,100);
      width += table->columnWidth(to);
    }
    height += table->rowHeight(ti);
  }
  height +=25;
  width +=45;
  table->resize(width, height);  
  table->setTopMargin(20);
  table->setLeftMargin(40);
  table->show();

  
  QProgressDialog *prog = new QProgressDialog (
					       "",
					       "&Cancel",
					       NGenerations,
					       0,
					       0,
					       true);
  prog->setProgress (0);
  
  for (g = 0; g < NGenerations; g++) {
    if (IsWindows&&g==3) {system("gpldisplay");}
    lcd->display(g+1);
    prog->setProgress (g+1);
    if (prog->wasCancelled()) break;
    
    if (NGenerations > 1) {
      genout << ": " << g+1 << endl;
    }
    
    InitializeGLA(value);	// All constraints are set to the InitialValue
    // Learn from observations: //
    
    for (c = 0; c < NObservations; c++) {
      	if (IsWindows&&!evolution && c==3) {system("gpldisplay");}	
		RandomRanking(value, Ranking);
		RandomCandidate(observation, GEN); // make an observation	
      
      o1 = eval.BestOutput(observation[0], Ranking, violates, GEN);
      for (i = 0; i < NConstraints; i++) {
 	if (violates[(observation[0]*NOutputs+o1)*NConstraints+i] > 
	    violates[(observation[0]*NOutputs+observation[1])*NConstraints+i]) {
// 	  value[i] += (Plasticity*(1000/(1000+(double)c)));
 	  value[i] += Plasticity;
  }
 	if (violates[(observation[0]*NOutputs+o1)*NConstraints+i] <
	    violates[(observation[0]*NOutputs+observation[1])*NConstraints+i]) {
// 	  value[i] -= (Plasticity*(1000/(1000+(double) c)));
 	  value[i] -= Plasticity;
  }
      }

      if (bi_learn) {
	i1 = eval.BestInput(observation[1], Ranking, violates, GEN);

	for (i = 0; i < NConstraints; i++) {
	  if (violates[(i1*NOutputs+observation[1])*NConstraints+i] >
	      violates[(observation[0]*NOutputs+observation[1])*NConstraints+i]) {
//	    value[i] += Plasticity*(1000/(1000+(double) c));
		value[i] += Plasticity;
	  }
	  if (violates[(i1*NOutputs+observation[1])*NConstraints+i] <
	      violates[(observation[0]*NOutputs+observation[1])*NConstraints+i]) {
//	    value[i] -= Plasticity*(1000/(1000+(double) c));
		value[i] -= Plasticity;
	  }
	}
      }
      if ((!evolution) && (c%StepSize == 0)) {    
	for (i = 0; i < NConstraints; i++) {
	  con_output << value[i] << "\t";      
	}
	con_output << endl;    
      }
    }
    // Display the learned grammar//
    
    if (evolution) {
      for (i = 0; i < NConstraints; i++) {
	con_output << value[i] << "\t";      
      }
      con_output << endl;    
    }
    
    // Generate test corpus with the learned grammar //
    
    for (i = 0; i < NInputs; i++) {
      n = 0;
      for (j = 0; j < NOutputs; j++) {
	if (GEN[i*NOutputs+j] == -1) {
	  TestOutput[i*NOutputs+j] = -1;
	} else {
	  TestOutput[i*NOutputs+j] = 0;
	  n += GEN[i*NOutputs+j];
	}
      }
      for (c1 = 0; c1 < n; c1++) {
	RandomRanking(value, Ranking);				
	o1 = eval.BestOutput(i, Ranking, violates, GEN);
	if (TestOutput[i*NOutputs+o1] > -1) TestOutput[i*NOutputs+o1]++;
      }
    }
    
    for (i = 0; i < NInputs; i++) {
      for (j = 0; j < NOutputs; j++) {
	GEN[i*NOutputs+j] = TestOutput[i*NOutputs+j];
	genout << GEN[i*NOutputs+j] << "\t";
	TestOutput[i*NOutputs+j] = 0;
      }
      genout << endl;
    }  
    genout << endl;
    display_gen(GEN, table);
  }
  con_output.close();
  QApplication::restoreOverrideCursor ();
}

//!Sets the ranking of all constraints to InitialValue.
void Evolotrun::InitializeGLA(vector<double> &constraintarray)
{
  int i;  
  
  for (i = 0; i < NConstraints; i++) {
    constraintarray[i] = InitialValue;
  }
}

void Evolotrun::GetGenerator(QString gen_file, vector<int> &GEN)
{
  char current_line[1000];
  char * word;
  int row, column;
    
  FILE * gen = fopen(gen_file, "r");	

  for (row=0;row<NInputs;row++) {
    fgets_no_comments(current_line, gen);
    word=strtok(current_line, " \t,;:");
    GEN[row*NOutputs+0] = strtol(word, NULL, 10);
    for (column=1;column<NOutputs;column++) {
      if ((word=strtok(NULL," \t,;:")) != NULL) {
	GEN[row*NOutputs+column] = strtol(word, NULL, 10);
      } else {
	GEN[row*NOutputs+column] = -1;
      }
    }
  }
  fclose(gen);
}

void Evolotrun::fgets_no_comments(char * line, FILE * input_file)
{
  char c='%';
  char * line_without_comments=0;
  int l=1;
  while(c=='%'||l==0) {
    fgets(line, LineLength, input_file);
    
    l = strlen(line);
    if (l == 1) continue;
    c = line[0];
    line_without_comments = strtok(line, "%");
  }
  strcpy(line, line_without_comments);
}


void Evolotrun::GetViolationPattern(QString starfile, vector<int> &violates)
{
  char current_line[1000];
  char * word;
  int input, output, constraint;
  FILE * infile;
  
  infile = fopen(starfile, "r");

  for (constraint=0;constraint<NConstraints;constraint++) {
    for (input=0; input<NInputs;input++) {
      fgets_no_comments(current_line, infile);
      word=strtok(current_line, " \t\n,;:");
      violates[(input*NOutputs+0)*NConstraints+constraint] = strtol(word, NULL, 10);
      for (output=1;output<NOutputs;output++) {
	if ((word=strtok(NULL," \t\n,;:")) != NULL) {
	  violates[(input*NOutputs+output)*NConstraints+constraint] = strtol(word, NULL, 10);
	} else {
	  violates[(input*NOutputs+output)*NConstraints+constraint] = 0;
	}
      }
    }
  }
  fclose(infile);
}
  

double Evolotrun::RandomNumber(void)
{
	double o;
	// returns a random number between 0 and 1.
	o = rand();
	return(o/((double) RAND_MAX+1));
}



void Evolotrun::RandomCandidate(int candidate[], vector<int> &generator)
{//could be done much simpler!
  double r;
  int i, i1=0, o, o1=0, n=0;
  
  r = RandomNumber();
  
  for (i = 0; i < NInputs; i++) {
    for (o = 0; o < NOutputs; o++) {
      if (generator[i*NOutputs+o] > -1) {
	n += generator[i*NOutputs+o];
      }
    }
  }// n = |GEN|
  
  r *= n;
  
  for (i = 0; i < NInputs; i++) {
    for (o = 0; o < NOutputs; o++) {
      if (r >= 0) {
	i1 = i;
	o1 = o;
      }
      if (generator[i*NOutputs+o] > -1) {
	r -= generator[i*NOutputs+o];
      }
    }
  }	
  
  candidate[0] = i1;
  candidate[1] = o1;
}

		
void Evolotrun::AddNoise(vector<double> &GrammarValues)
{
	int i;
	for (i = 0; i < NConstraints; i++) {
	  ActualValues[i] = GrammarValues[i] + Noise*cndev(RandomNumber());
	} // Add normal distributed noise value to value of each constraint
}


void Evolotrun::RandomRanking(vector<double> &GrammarValues, vector<int> &NewRanking)
{
  int i;
  
  AddNoise(GrammarValues);
  for (i = 0; i < NConstraints; i++) {
    NewRanking[i] = i;
  } // Initialize the constraint ranking	
  gjsort(NewRanking);
  // sort constraints according to their current value	
}

void Evolotrun::gjsort(vector<int> &bag)
{
  quicksort(bag, 1, bag.size());
}

void Evolotrun::quicksort(vector<int> &bag, int p, int size_of_bag)
{
  int q;
  
  if (p < size_of_bag) {
    q = partition(bag, p, size_of_bag);
    quicksort(bag, p, q+1);
    quicksort(bag, q+2, size_of_bag);
  }
}

int Evolotrun::partition(vector<int> &bag, int p, int r)
{
  int x, i, j, b;
  
  x = bag[p-1];
  i = p-1;
  j = r-1;
  
  while (1==1) {
    while (ActualValues[bag[i]] < ActualValues[x]) i++;
    while (ActualValues[bag[j]] > ActualValues[x]) j--;
    if (i < j) {
      b = bag[i];
      bag[i] = bag[j];
      bag[j] = b;
    } else {
      break;
    }
  }
  return j;
}

void Evolotrun::display_gen(vector<int> &GEN, QTable *t)
{
  int r=0, c=0;
  for (r=0;r<NInputs;r++) {
    for (c=0;c<NOutputs;c++) {
      QString s;
      t->setText(r,c, "  "+s.setNum(GEN[r*NOutputs+c])+" ");
    }
  }
}
