/***************************************************************************
                      evolotimpl.cpp  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file evolotimpl.cpp
   \brief Implementation of interface between GUI and actual program of evolOT

   Contains the implementation of the class 'Evolot', a sub-class of the GUI class
   'evolotBase'. This class stores the information from the GUI in its members and 
   passes them on to 'Evolotrun'.
*/


#include "evolotimpl.h"


Evolot::Evolot( QWidget* parent, const char* name,
		bool modal, WFlags f) :
  evolotBase( parent, name, modal, f )
{
  NGenerations = 100;
  NEquivalenceClasses = 1;
  NObservations = 20000;
  InitialValue = 0;
  Noise = 20;
  Plasticity = 10;
  StepSize = 100;
  bi_eval = true;
  bi_learn = true;
  evolution = true;

	QFile configfile(".evolotrc");
	configfile.open(IO_ReadOnly);
	QTextStream cfg(&configfile);
	//read content into map
	QMap<QString,QString> cfgmap;
	QString line;
	while(line=cfg.readLine()) {
	cfg.skipWhiteSpace();
    if (line.isEmpty()) continue;
		cfgmap[line.left(line.find(":"))] = line.mid(line.find(":")+1,line.length());
		}
	configfile.close();

	
	if (cfgmap.contains("[InitialValue]")) {
		InitialValue = cfgmap["[InitialValue]"].toDouble();
		initialSpinBox->setValue((int) InitialValue);
		}

	if (cfgmap.contains("[NConstraints]")) {
		NConstraints = cfgmap["[NConstraints]"].toInt();
		constraintSpinBox->setValue(NConstraints);
		}
	if (cfgmap.contains("[NEquivalenceClasses]")) {
		NEquivalenceClasses = cfgmap["[NEquivalenceClasses]"].toInt();
		eqSpinBox->setValue(NEquivalenceClasses);
		}
	if (cfgmap.contains("[NGenerations]")) {
		NGenerations = cfgmap["[NGenerations]"].toInt();
		generationSpinBox->setValue(NGenerations);
		}
	if (cfgmap.contains("[NInputs]")) {
		NInputs = cfgmap["[NInputs]"].toInt();
		inputSpinBox->setValue(NInputs);
		}
	if (cfgmap.contains("[NObservations]")) {
		NObservations = cfgmap["[NObservations]"].toInt();
		observationSpinBox->setValue(NObservations);
		}
	if (cfgmap.contains("[NOutputs]")) {
		NOutputs = cfgmap["[NOutputs]"].toInt();
		outputSpinBox->setValue(NOutputs);
		}
	if (cfgmap.contains("[Noise]")) {
		Noise = cfgmap["[Noise]"].toDouble();
		noiseSpinBox->setValue((int) (Noise*10));
		}
	if (cfgmap.contains("[Plasticity]")) {
		Plasticity = cfgmap["[Plasticity]"].toDouble();
		plasticitySpinBox->setValue((int) (Plasticity*1000));
		}
	if (cfgmap.contains("[StepSize]")) {
		StepSize = cfgmap["[StepSize]"].toInt();
		stepsizeSpinBox->setValue(StepSize);
		}
	if (cfgmap.contains("[bi_eval]")) {
		bi_eval = cfgmap["[bi_eval]"].toInt();
		bievalCheckBox->setChecked((bool) bi_eval);
		}
	if (cfgmap.contains("[bi_learn]")) {
		bi_learn = cfgmap["[bi_learn]"].toInt();
		bilearningCheckBox->setChecked((bool) bi_learn);
		}
	if (cfgmap.contains("[con_outputfile]")) {
		con_outputfile = cfgmap["[con_outputfile]"];
		rankingLineEdit->setText(con_outputfile);
		}
	if (cfgmap.contains("[evolution]")) {
		evolution = cfgmap["[evolution]"].toInt();
		if (evolution) {
			EvolutionComboBox->setCurrentText( "Evolution");
			} else {
			EvolutionComboBox->setCurrentText( "Learning" );
			}
		}
	if (cfgmap.contains("[gen_outputfile]")) {
		gen_outputfile = cfgmap["[gen_outputfile]"];
		freqLineEdit->setText(gen_outputfile);
		}
	if (cfgmap.contains("[genfile]")) {
		genfile = cfgmap["[genfile]"];
		genLineEdit->setText(genfile);
		}
	if (cfgmap.contains("[scriptfile]")) {
		scriptfile = cfgmap["[scriptfile]"];
		scriptLineEdit->setText(scriptfile);
		}
	if (cfgmap.contains("[starfile]")) {
		starfile = cfgmap["[starfile]"];
		conLineEdit->setText(starfile);
		}
  
  QTable *t = new QTable(0, "Corpus");
  table = t;
  
  QLCDNumber *l = new QLCDNumber(5, 0, "Generations");
  lcd = l;
}

Evolot::~Evolot()
{
	delete (table);
	delete (lcd);
}

void Evolot::bieval_slot(bool b){
  bi_eval = b;
  }
void Evolot::bilearn_slot(bool b){
  bi_learn = b;
}

void Evolot::evolution_slot(const QString& s)
{
  if (s=="Evolution") {
    evolution = true;
    generationSpinBox->setEnabled(true);
    stepsizeSpinBox->setEnabled(false);
    generationSpinBox->setValue(100);
    NGenerations = 100;
  } else {
    evolution = false;
    generationSpinBox->setEnabled(false);
    stepsizeSpinBox->setEnabled(true);
    NGenerations = 1;
  }
}

void Evolot::stepsize_slot(int n)
{
  StepSize = n;
}

void Evolot::freqfileDialogslot(){
  gen_outputfile = QFileDialog::getOpenFileName( QString::null, QString::null, this , "frequencies file", "frequencies file");
  freqLineEdit->validateAndSet(gen_outputfile,0, 0, 0);
}
void Evolot::freqfile_slot(const QString& f){
  gen_outputfile = f;
}
void Evolot::genfileDialogslot(){
  genfile = QFileDialog::getOpenFileName( QString::null, QString::null, this, "open GEN file", "open GEN file" );
  genLineEdit->validateAndSet(genfile,0, 0, 0);
}
void Evolot::genfile_slot(const QString& f){
  genfile = f;
}
void Evolot::rankingfileDialogslot(){
  con_outputfile = QFileDialog::getOpenFileName( QString::null, QString::null, this,  "rankings file", "rankings file");
  rankingLineEdit->validateAndSet(con_outputfile,0, 0, 0);
}
void Evolot::rankingfile_slot(const QString& f){
  con_outputfile = f;
}
void Evolot::scriptfileDialogslot(){
  scriptfile = QFileDialog::getOpenFileName( QString::null, QString::null, this,  "open scriptfile", "open scriptfile");
  scriptLineEdit->validateAndSet(scriptfile,0, 0, 0);
}
void Evolot::scriptfile_slot(const QString& f){
  scriptfile = f;
}
void Evolot::starfileDialogslot(){
  starfile = QFileDialog::getOpenFileName( QString::null, QString::null, this,  "open starfile", "open starfile");
  conLineEdit->validateAndSet(starfile,0, 0, 0);
}
void Evolot::starfile_slot(const QString& f){
  starfile = f;
}
void Evolot::initial_slot(int n){
  InitialValue = n;
}
void Evolot::nconstraints_slot(int n){
  NConstraints = n;
}
void Evolot::neq_slot(int n){
  NEquivalenceClasses = n;
}
void Evolot::ngenerations_slot(int n){
  NGenerations = n;
}
void Evolot::ninput_slot(int n){
  NInputs = n;
}
void Evolot::nobservations_slot(int n){
  NObservations = n;
}
void Evolot::noise_slot(int n){
  Noise = n;
}
void Evolot::noutput_slot(int n){
  NOutputs = n;
}
void Evolot::plasticity_slot(int n){
  Plasticity = n;
}

void Evolot::compile_slot()
{
  StarmakerRun smrun;
  smrun.NInputs = NInputs;
  smrun.NOutputs = NOutputs;
  smrun.NConstraints = NConstraints;
  smrun.scriptfilename = scriptfile;
  smrun.starfilename = starfile;
  smrun.con_outputfile = con_outputfile;
  smrun.run();
}


void Evolot::run_slot(){

  	QFile configfile(".evolotrc");
	configfile.open(IO_WriteOnly);
	QTextStream cfg(&configfile);
	cfg << "[InitialValue]:" << InitialValue << endl;
	cfg << "[NConstraints]:" << NConstraints << endl;
	cfg << "[NEquivalenceClasses]:" << NEquivalenceClasses << endl;
	cfg << "[NGenerations]:" << NGenerations << endl;
	cfg << "[NInputs]:" << NInputs << endl;
	cfg << "[NObservations]:" << NObservations << endl;
	cfg << "[NOutputs]:" << NOutputs << endl;
	cfg << "[Noise]:" << Noise << endl;
	cfg << "[Plasticity]:" << Plasticity << endl;
	cfg << "[StepSize]:" << StepSize << endl;
	cfg << "[bi_eval]:" << bi_eval << endl;
	cfg << "[bi_learn]:" << bi_learn << endl;
	cfg << "[con_outputfile]:" << con_outputfile << endl;
	cfg << "[evolution]:" << evolution << endl;
	cfg << "[gen_outputfile]:" << gen_outputfile << endl;
	cfg << "[genfile]:" << genfile << endl;
	cfg << "[scriptfile]:" << scriptfile << endl;
	cfg << "[starfile]:" << starfile << endl;
	configfile.close();

  Evolotrun e;

  e.NInputs = NInputs; 
  e.NOutputs = NOutputs; 
  e.NConstraints = NConstraints; 
  e.NGenerations = NGenerations; 
  e.NEquivalenceClasses = NEquivalenceClasses; 
  e.NObservations = NObservations; 
  e.InitialValue = InitialValue; 
  e.Noise = Noise * 0.1; 
  e.Plasticity = Plasticity * 0.001; 
  e.StepSize = StepSize;
  e.genfile = genfile; 
  e.con_outputfile = con_outputfile; 
  e.gen_outputfile = gen_outputfile; 
  e.starfile = starfile; 
  e.bi_eval = bi_eval; 
  e.bi_learn = bi_learn;
  e.evolution = evolution;
  e.table = table;
  e.lcd = lcd;
  
  e.run();
}

