/***************************************************************************
                       starmakerrun.cpp  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file starmakerrun.cpp
   \brief contains implementation of the StarmakerRun class
*/

#include "starmakerrun.h"

void StarmakerRun::run()
{
	bool IsWindows=false;
  
  QString command, cname, expr, n_stars;
  int c, i, o, stars, cno=1;
  vector <int> matrix((NConstraints+1)*(NInputs+1)*(NOutputs+1));
  map<QString,int> constraintnumber;
  map<int,QString> constraintname;
        
  QFile in(scriptfilename);
  in.open(IO_ReadOnly);
  QTextStream instream(&in);

    
  // initialize the matrix with 0 in every slot
  for (c=1;c<=NConstraints;c++) {
    for (i=1;i<=NInputs;i++) {
      for(o=1;o<=NOutputs;o++) {
	matrix[(c*NInputs+i)*NOutputs+o] = 0;
      }
    }
  }
  
  // get a command line from infile and split it into "constraint number", 
  // "command body" and "number of stars"

  while (command = instream.readLine()) {	  
    instream.skipWhiteSpace();

    if (command.isEmpty()) continue;
    cname = command.left(command.find(":"));
    expr = command.mid(command.find(":")+1,command.find(";")-command.find(":")-1);

    n_stars = command.mid(command.find(";")+1,command.length());

    if (constraintnumber[cname] == 0) {
      constraintnumber[cname] = cno;
      constraintname[cno++] = cname;
    }
                
    if (!n_stars.isEmpty()) {
      stars = n_stars.toInt();
    } else {
      stars = 1;
    }
    if (stars < 1) stars = 1; //dos compatibility    
    
    // update the matrix
    if (cno > NConstraints+1) {
      QMessageBox::warning(0, "Warning", "Scriptfile contains too many constraints!");
    } else {
      for (i=1;i<=NInputs;i++) {
				for (o=1;o<=NOutputs;o++) {
				  if (evaluate(expr,i,o)) {
				    matrix[(constraintnumber[cname]*NInputs+i)*NOutputs+o] += stars;
				  }
				}
      }
    }  
  }

  in.close();
  // write content of matrix into outfile
  ofstream out(starfilename);
  for (c=1;c<=NConstraints;c++) {
    out << "% ";
    (constraintname[c].length() > 0)?out << constraintname[c] << endl:out << c << endl;
    for (i=1;i<=NInputs;i++) {
      for (o=1;o<=NOutputs;o++) {
				out << matrix[(c*NInputs+i)*NOutputs+o] << "\t";
      }
      out << endl;
    }
    out << endl << endl;
  }
  out.close();

	if (IsWindows) {
	  QFile gpl(".gplscript");
	  gpl.open(IO_WriteOnly);
	  QTextStream gplstream(&gpl);
	  gplstream << "set key below" << endl;
	  gplstream << "set yrange [-9:9]" << endl;
	  gplstream << "plot";
	  for (c=1;c<=NConstraints;c++) {
    	gplstream << " \"" << con_outputfile << "\" using 0:" << c;
     	gplstream << " smooth bezier title \"" << constraintname[c] << "\"";
      if (c<NConstraints) {
      	gplstream << ", ";
       }
      }
     gplstream << endl << "pause -1" << endl;
     gplstream << "reread" << endl;
    gpl.close();
    }
}

bool StarmakerRun::evaluate(QString expr, int i, int o) {
  int cursor, lhs=0, rhs;

  // else find first top-level '->', evaluate subexpressions
  if ((cursor = find_toplevel(expr, "->"))>=0) {
    return (!evaluate(expr.mid(0,cursor),i,o)) | (evaluate(expr.mid(cursor+2,expr.length()),i,o));
  }

  // find first top-level '|', evaluate subexpressions
  
  if ((cursor = find_toplevel(expr, "|"))>=0) {
    return evaluate(expr.mid(0,cursor),i,o) | evaluate(expr.mid(cursor+1,expr.length()),i,o);
  }
  
  // else find first top-level '&', evaluate subexpressions
  if ((cursor = find_toplevel(expr, "&"))>=0) {
    return evaluate(expr.mid(0,cursor),i,o) & (evaluate(expr.mid(cursor+1,expr.length()),i,o));
  }

  // else find first top-level '!', evaluate subexpression
  if ((cursor = find_toplevel(expr, "!"))>=0) {
    if (expr[cursor+1] != '=') return !evaluate(expr.mid(cursor+1,expr.length()),i,o);
  }
  
  // else find first top-level '(' and ')', evaluate subexpression
  if (contains(expr,"(")) {
    return evaluate(expr.mid(expr.find("(")+1,expr.findRev(")")-expr.find("(")-1),i,o);
  }
  
  if((cursor = find_toplevel(expr, ">=")) >= 0) {
    if (contains(expr.mid(0,cursor),"i")) lhs = i;
    if (contains(expr.mid(0,cursor),"o")) lhs = o;
    rhs = expr.mid(cursor+2,expr.length()).toInt();
    return (lhs >= rhs);
  }
  
  if((cursor = find_toplevel(expr, "<=")) >= 0) {
    if (contains(expr.mid(0,cursor),"i")) lhs = i;
    if (contains(expr.mid(0,cursor),"o")) lhs = o;
    rhs = expr.mid(cursor+2,expr.length()).toInt();
    return (lhs <= rhs);
  }
  
  if((cursor = find_toplevel(expr, "==")) >= 0) {
    if (contains(expr.mid(0,cursor),"i")) lhs = i;
    if (contains(expr.mid(0,cursor),"o")) lhs = o;
    rhs = expr.mid(cursor+2,expr.length()).toInt();
    return (lhs == rhs);
  }
  
  if((cursor = find_toplevel(expr, "!=")) >= 0) {
    if (contains(expr.mid(0,cursor),"i")) lhs = i;
    if (contains(expr.mid(0,cursor),"o")) lhs = o;
    rhs = expr.mid(cursor+2,expr.length()).toInt();
    return (lhs != rhs);
  }
  
  if((cursor = find_toplevel(expr, ">")) >= 0) {
    if (contains(expr.mid(0,cursor),"i")) lhs = i;
    if (contains(expr.mid(0,cursor),"o")) lhs = o;
    rhs = expr.mid(cursor+1,expr.length()).toInt();
    return (lhs > rhs);
  }
  
  if((cursor = find_toplevel(expr, "<")) >= 0) {
    if (contains(expr.mid(0,cursor),"i")) lhs = i;
    if (contains(expr.mid(0,cursor),"o")) lhs = o;
    rhs = expr.mid(cursor+1,expr.length()).toInt();
    return (lhs < rhs);
  }
  
  //  cerr << "Syntax error!\n";
  
  return FALSE;
}

bool StarmakerRun::contains(QString s1, QString s2)
{
  if (0<= (int) s1.find(s2)&(int) s1.find(s2)<(int) s1.length()) return TRUE;
  return FALSE;
}


// Returns the starting position of the first occurrence of "op" in "expr"
// that is not embedded in parentheses. If there is no such occurrence,
// returns -1.
int StarmakerRun::find_toplevel(QString expr, QString op)
{
  vector<int> st(expr.length());
  int n=0, c;
  for (c=0;c<(int) expr.length();c++) {
    st[c] = n;
    if (expr[c] == '(') n++;
    if (expr[c] == ')') n--;
  }
  
  return find_toplevel4(expr, op, 0, st);
}

//same as above, except that search starts at position pos
int StarmakerRun::find_toplevel4(QString expr, QString op, int pos, vector<int> st)
{
  int cursor;
  
  if (pos >= (int) expr.length()) return -1;
  if (contains(expr.mid(pos,expr.length()), op)) {
    cursor = pos+expr.mid(pos,expr.length()).find(op);
  } else return -1;
  
  if (st[cursor] == 0) {
    return cursor;
  } 
  if (contains(expr.mid(cursor,expr.length()), op)) {
      return find_toplevel4(expr, op, cursor+1, st);
    }
  return -1;
}
