#ifndef STARMAKERRUN_H
#define STARMAKERRUN_H
/***************************************************************************
                       starmakerrun.h  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file starmakerrun.h
   \brief contains header of the StarmakerRun class
*/

#include <iostream>
#include <fstream>
#include <map>
#include <vector>
#include <qtextstream.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qstring.h>
using namespace std;


//! the compiler that translates constraint descriptions into tables with violation patterns
/*! reads a script from StarmakerRun::scriptfile, translates it into a 3d-matrix of 
    integers (StarmakerRun::NConstraints x StarmakerRun::NInputs x StarmakerRun::NOutputs)
    and saves this matrix alongside with the names of the constraints in 
    Starmakerrun::starfilename
*/
class StarmakerRun
{
 public:
  int NInputs;
  int NOutputs;
  int NConstraints;
  QString scriptfilename;
  QString starfilename;
  QString con_outputfile;

 public:
  void run();
  
 private:

  //! 'true' if 'expr' is true of (i,o)
  /*! 'expr' is a arithmetic formula over the variables "i" and "o". The function evaluates
    as 'true' if 'expr' is true of the values of 'i' and 'o', false else.
  */
  bool evaluate(QString expr, int i, int o);
  
  //! 'true' iff 's2' is a substring of 's1'
  bool contains(QString s1, QString s2);
  
  //! returns the index of the first toplevel occurrence of 'op' in 'expr'
  /*! where "toplevel" means "not in the scope of parentheses. Returns -1 if
    'op' does not occur in 'expr'.
  */
  int find_toplevel(QString expr, QString op);
  
  //! returns the index of the first toplevel occurrence of 'op' in 'expr' after position 'pos'
  /*! where 'st' is an integer array such that 'st[i] =n' iff 'expr[i]' is embedded in 'n' bracket 
    pairs.
  */
  int find_toplevel4(QString expr, QString op, int pos, vector<int> st);
}
;

#endif

